@@ -49,17 +49,19 @@
 #include "src/common/net.h"
 #include "src/common/persist_conn.h"
 #include "src/common/slurm_protocol_pack.h"
+#include "src/common/slurm_time.h"
 #include "src/common/slurmdbd_defs.h"
 #include "src/common/slurmdbd_pack.h"
 #include "src/common/xsignal.h"
 #include "src/interfaces/auth.h"
-#include "src/interfaces/tls.h"
+#include "src/interfaces/conn.h"
 
 #define MAX_THREAD_COUNT 100
 
 typedef struct {
 	void *arg;
-	persist_conn_t *conn;
+	persist_conn_t *pcon;
+	int fd;
 	int thread_loc;
 	pthread_t thread_id;
 } persist_service_conn_t;
@@ -73,18 +75,6 @@ static time_t          shutdown_time = 0;
 static buf_t *_slurm_persist_recv_msg(persist_conn_t *persist_conn,
 				      bool reopen);
 
-/* Return time in msec since "start time" */
-static int _tot_wait (struct timeval *start_time)
-{
-	struct timeval end_time;
-	int msec_delay;
-
-	gettimeofday(&end_time, NULL);
-	msec_delay =   (end_time.tv_sec  - start_time->tv_sec ) * 1000;
-	msec_delay += ((end_time.tv_usec - start_time->tv_usec + 500) / 1000);
-	return msec_delay;
-}
-
 /* Return true if communication failure should be logged. Only log failures
  * every 10 minutes to avoid filling logs */
 static bool _comm_fail_log(persist_conn_t *persist_conn)
@@ -115,13 +105,21 @@ static bool _conn_readable(persist_conn_t *persist_conn)
 
 	xassert(persist_conn->shutdown);
 
-	ufds.fd     = persist_conn->fd;
+	/*
+	 * The tls layer may already have data buffered, which could lead to
+	 * poll blocking indefinitely.
+	 */
+	if (conn_g_peek(persist_conn->conn))
+		return true;
+
+	ufds.fd = conn_g_get_fd(persist_conn->conn);
 	ufds.events = POLLIN;
 	while (!(*persist_conn->shutdown)) {
 		if (persist_conn->timeout) {
 			struct timeval tstart;
 			gettimeofday(&tstart, NULL);
-			time_left = persist_conn->timeout - _tot_wait(&tstart);
+			time_left = persist_conn->timeout -
+				    timeval_tot_wait(&tstart);
 		} else
 			time_left = -1;
 		rc = poll(&ufds, 1, time_left);
@@ -130,27 +128,27 @@ static bool _conn_readable(persist_conn_t *persist_conn)
 		if (rc == -1) {
 			if ((errno == EINTR) || (errno == EAGAIN)) {
 				debug3("%s: retrying poll for fd %d: %m",
-					__func__, persist_conn->fd);
+					__func__, ufds.fd);
 				continue;
 			}
 			error("%s: poll error for fd %d: %m",
-			      __func__, persist_conn->fd);
+			      __func__, ufds.fd);
 			return false;
 		}
 		if (rc == 0) {
 			debug("%s: poll for fd %d timeout after %d msecs of total wait %d msecs.",
-			      __func__, persist_conn->fd, time_left,
+			      __func__, ufds.fd, time_left,
 			      persist_conn->timeout);
 			return false;
 		}
 		if (ufds.revents & POLLHUP) {
 			log_flag(NET, "%s: persistent connection for fd %d closed",
-				 __func__, persist_conn->fd);
+				 __func__, ufds.fd);
 			return false;
 		}
 		if (ufds.revents & POLLNVAL) {
 			error("%s: persistent connection for fd %d is invalid",
-			       __func__, persist_conn->fd);
+			       __func__, ufds.fd);
 			return false;
 		}
 		if (ufds.revents & POLLERR) {
@@ -167,7 +165,7 @@ static bool _conn_readable(persist_conn_t *persist_conn)
 		}
 		if ((ufds.revents & POLLIN) == 0) {
 			error("%s: persistent connection for fd %d missing POLLIN flag with revents 0x%"PRIx64,
-			      __func__, persist_conn->fd, (uint64_t) ufds.revents);
+			      __func__, ufds.fd, (uint64_t) ufds.revents);
 			return false;
 		}
 		if (ufds.revents == POLLIN) {
@@ -180,14 +178,14 @@ static bool _conn_readable(persist_conn_t *persist_conn)
 	}
 
 	debug("%s: shutdown request detected for fd %d",
-	      __func__, persist_conn->fd);
+	      __func__, ufds.fd);
 	return false;
 }
 
 static void _destroy_persist_service(persist_service_conn_t *persist_service)
 {
 	if (persist_service) {
-		slurm_persist_conn_destroy(persist_service->conn);
+		slurm_persist_conn_destroy(persist_service->pcon);
 		xfree(persist_service);
 	}
 }
@@ -205,7 +203,8 @@ static void _persist_free_msg_members(persist_conn_t *persist_conn,
 		slurm_free_msg_data(persist_msg->msg_type, persist_msg->data);
 }
 
-static int _process_service_connection(persist_conn_t *persist_conn, void *arg)
+static int _process_service_connection(persist_conn_t *persist_conn, int fd,
+				       void *arg)
 {
 	uint32_t nw_size = 0, msg_size = 0;
 	char *msg_char = NULL;
@@ -213,39 +212,48 @@ static int _process_service_connection(persist_conn_t *persist_conn, void *arg)
 	bool first = true, fini = false;
 	buf_t *buffer = NULL;
 	int rc = SLURM_SUCCESS;
+	conn_args_t conn_args = {
+		.input_fd = fd,
+		.output_fd = fd,
+		.mode = CONN_SERVER,
+	};
 
 	xassert(persist_conn->callback_proc);
 	xassert(persist_conn->shutdown);
 
 	log_flag(NET, "%s: Opened connection %d from %s",
-		 __func__, persist_conn->fd, persist_conn->rem_host);
+		 __func__, fd, persist_conn->rem_host);
 
 	if (persist_conn->flags & PERSIST_FLAG_ALREADY_INITED)
 		first = false;
 
+	if (first && !(persist_conn->conn = conn_g_create(&conn_args))) {
+		error("%s: conn_g_create() failed negotiation, closing connection %d(%s)",
+		      __func__, fd, persist_conn->rem_host);
+		(void) close(fd);
+		return SLURM_ERROR;
+	}
+	conn_g_set_graceful_shutdown(persist_conn->conn, true);
+
 	while (!(*persist_conn->shutdown) && !fini) {
 		if (!_conn_readable(persist_conn))
 			break;		/* problem with this socket */
 
-		if (first)
-			msg_read = read(persist_conn->fd, &nw_size,
-					sizeof(nw_size));
-		else
-			msg_read = tls_g_recv(persist_conn->tls_conn, &nw_size,
-					      sizeof(nw_size));
+		msg_read = conn_g_recv(persist_conn->conn, &nw_size,
+				       sizeof(nw_size));
 		if (msg_read == 0)	/* EOF */
 			break;
 		if (msg_read != sizeof(nw_size)) {
 			error("Could not read msg_size from connection %d(%s) uid(%u)",
-			      persist_conn->fd, persist_conn->rem_host,
+			      fd, persist_conn->rem_host,
 			      persist_conn->auth_uid);
 			break;
 		}
 		msg_size = ntohl(nw_size);
 		if ((msg_size < 2) || (msg_size > MAX_MSG_SIZE)) {
 			error("Invalid msg_size (%u) from connection %d(%s) uid(%u)",
-			      msg_size, persist_conn->fd,
-			      persist_conn->rem_host, persist_conn->auth_uid);
+			      msg_size, fd, persist_conn->rem_host,
+			      persist_conn->auth_uid);
 			break;
 		}
 
@@ -254,16 +262,11 @@ static int _process_service_connection(persist_conn_t *persist_conn, void *arg)
 		while (msg_size > offset) {
 			if (!_conn_readable(persist_conn))
 				break;		/* problem with this socket */
-			if (first)
-				msg_read = read(persist_conn->fd,
-						(msg_char + offset),
-						(msg_size - offset));
-			else
-				msg_read = tls_g_recv(persist_conn->tls_conn,
-						      (msg_char + offset),
-						      (msg_size - offset));
+			msg_read = conn_g_recv(persist_conn->conn,
+					       (msg_char + offset),
+					       (msg_size - offset));
 			if (msg_read <= 0) {
-				error("read(%d): %m", persist_conn->fd);
+				error("read(%d): %m", fd);
 				break;
 			}
 			offset += msg_read;
@@ -286,8 +289,7 @@ static int _process_service_connection(persist_conn_t *persist_conn, void *arg)
 				    (rc != ACCOUNTING_TRES_CHANGE_DB) &&
 				    (rc != ACCOUNTING_NODES_CHANGE_DB)) {
 					error("Processing last message from connection %d(%s) uid(%u)",
-					      persist_conn->fd,
-					      persist_conn->rem_host,
+					      fd, persist_conn->rem_host,
 					      persist_conn->auth_uid);
 					if (rc == ESLURM_ACCESS_DENIED ||
 					    rc == SLURM_PROTOCOL_VERSION_ERROR)
@@ -312,8 +314,7 @@ static int _process_service_connection(persist_conn_t *persist_conn, void *arg)
 				if (persist_conn->rem_port)
 					log_flag(NET, "%s: Problem sending response to connection host:%s fd:%d uid:%u",
 						 __func__,
-						 persist_conn->rem_host,
-						 persist_conn->fd,
+						 persist_conn->rem_host, fd,
 						 persist_conn->auth_uid);
 				fini = true;
 			}
@@ -322,8 +323,7 @@ static int _process_service_connection(persist_conn_t *persist_conn, void *arg)
 	}
 
 	log_flag(NET, "%s: Closed connection host:%s fd:%d uid:%u",
-		 __func__, persist_conn->rem_host, persist_conn->fd,
-		 persist_conn->auth_uid);
+		 __func__, persist_conn->rem_host, fd, persist_conn->auth_uid);
 
 	return rc;
 }
@@ -333,11 +333,11 @@ static void *_service_connection(void *arg)
 	persist_service_conn_t *service_conn = arg;
 
 	xassert(service_conn);
-	xassert(service_conn->conn);
+	xassert(service_conn->pcon);
 
 #if HAVE_SYS_PRCTL_H
 	char *name = xstrdup_printf("p-%s",
-				    service_conn->conn->cluster_name);
+				    service_conn->pcon->cluster_name);
 	if (prctl(PR_SET_NAME, name, NULL, NULL, NULL) < 0) {
 		error("%s: cannot set my name to %s %m", __func__, name);
 	}
@@ -346,13 +346,14 @@ static void *_service_connection(void *arg)
 
 	service_conn->thread_id = pthread_self();
 
-	_process_service_connection(service_conn->conn, service_conn->arg);
+	_process_service_connection(service_conn->pcon, service_conn->fd,
+				    service_conn->arg);
 
-	if (service_conn->conn->callback_fini)
-		(service_conn->conn->callback_fini)(service_conn->arg);
+	if (service_conn->pcon->callback_fini)
+		(service_conn->pcon->callback_fini)(service_conn->arg);
 	else
 		log_flag(NET, "%s: Persist connection from cluster %s has disconnected",
-			 __func__, service_conn->conn->cluster_name);
+			 __func__, service_conn->pcon->cluster_name);
 
 	/* service_conn is freed inside here */
 	slurm_persist_conn_free_thread_loc(service_conn->thread_loc);
@@ -383,9 +384,6 @@ extern void slurm_persist_conn_recv_server_init(void)
 
 	shutdown_time = 0;
 
-	(void) pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
-	(void) pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);
-
 	/* Prepare to catch SIGUSR1 to interrupt accept().
 	 * This signal is generated by the slurmdbd signal
 	 * handler thread upon receipt of SIGABRT, SIGINT,
@@ -418,15 +416,23 @@ extern void slurm_persist_conn_recv_server_fini(void)
 			pthread_t thread_id =
 				persist_service_conn[i]->thread_id;
 
-			/* Let go of lock in case the persistent connection
+			/*
+			 * Let go of lock in case the persistent connection
 			 * thread is cleaning itself up.
 			 * slurm_persist_conn_free_thread_loc() may be trying to
 			 * remove itself but could be waiting on the
-			 * thread_count mutex which this has locked. */
+			 * thread_count mutex which this has locked.
+			 * After joining persist_service_conn[i] could be NULL
+			 */
 			slurm_mutex_unlock(&thread_count_lock);
 			slurm_thread_join(thread_id);
 			slurm_mutex_lock(&thread_count_lock);
 		}
+		if (persist_service_conn[i] && persist_service_conn[i]->pcon) {
+			void *conn = persist_service_conn[i]->pcon->conn;
+			conn_g_set_graceful_shutdown(conn, false);
+		}
+
 		_destroy_persist_service(persist_service_conn[i]);
 		persist_service_conn[i] = NULL;
 	}
@@ -434,7 +440,8 @@ extern void slurm_persist_conn_recv_server_fini(void)
 }
 
 extern void slurm_persist_conn_recv_thread_init(persist_conn_t *persist_conn,
-						int thread_loc, void *arg)
+						int fd, int thread_loc,
+						void *arg)
 {
 	persist_service_conn_t *service_conn;
 
@@ -443,14 +450,15 @@ extern void slurm_persist_conn_recv_thread_init(persist_conn_t *persist_conn,
 	if (thread_loc < 0)
 		return;
 
-	service_conn = xmalloc(sizeof(persist_service_conn_t));
-
 	slurm_mutex_lock(&thread_count_lock);
-	persist_service_conn[thread_loc] = service_conn;
+	service_conn = persist_service_conn[thread_loc];
+	xassert(service_conn);
+	xassert(!service_conn->arg);
 	slurm_mutex_unlock(&thread_count_lock);
 
 	service_conn->arg = arg;
-	service_conn->conn = persist_conn;
+	service_conn->pcon = persist_conn;
+	service_conn->fd = fd;
 	service_conn->thread_loc = thread_loc;
 
 	persist_conn->timeout = 0; /* If this isn't zero we won't wait forever
@@ -480,6 +488,9 @@ extern int slurm_persist_conn_wait_for_thread_loc(void)
 			for (i=0; i<MAX_THREAD_COUNT; i++) {
 				if (persist_service_conn[i])
 					continue;
+
+				persist_service_conn[i] =
+					xmalloc(sizeof(persist_service_conn_t));
 				rc = i;
 				break;
 			}
@@ -536,16 +547,17 @@ extern void slurm_persist_conn_free_thread_loc(int thread_loc)
 static int _open_persist_conn(persist_conn_t *persist_conn)
 {
 	slurm_addr_t addr;
+	int fd;
 
 	xassert(persist_conn);
 	xassert(persist_conn->rem_host);
 	xassert(persist_conn->rem_port);
 	xassert(persist_conn->cluster_name);
 
-	if (persist_conn->fd > 0)
-		fd_close(&persist_conn->fd);
-	else
-		persist_conn->fd = -1;
+	if (persist_conn->conn) {
+		conn_g_destroy(persist_conn->conn, true);
+		persist_conn->conn = NULL;
+	}
 
 	if (!persist_conn->inited)
 		persist_conn->inited = true;
@@ -561,7 +573,8 @@ static int _open_persist_conn(persist_conn_t *persist_conn)
 		persist_conn->timeout = slurm_conf.msg_timeout * 1000;
 
 	slurm_set_addr(&addr, persist_conn->rem_port, persist_conn->rem_host);
-	if ((persist_conn->fd = slurm_open_msg_conn(&addr)) < 0) {
+
+	if (!(persist_conn->conn = slurm_open_msg_conn(&addr, NULL))) {
 		if (_comm_fail_log(persist_conn)) {
 			if (persist_conn->flags & PERSIST_FLAG_SUPPRESS_ERR)
 				log_flag(NET, "%s: failed to open persistent connection (with error suppression active) to host:%s:%d: %m",
@@ -574,8 +587,17 @@ static int _open_persist_conn(persist_conn_t *persist_conn)
 		}
 		return SLURM_ERROR;
 	}
-	fd_set_nonblocking(persist_conn->fd);
-	net_set_keep_alive(persist_conn->fd);
+
+	/*
+	 * Peer will be waiting on conn_g_recv(), and they will need to know if
+	 * connection was intentionally closed or if an error occurred.
+	 */
+	conn_g_set_graceful_shutdown(persist_conn->conn, true);
+
+	fd = conn_g_get_fd(persist_conn->conn);
+
+	fd_set_nonblocking(fd);
+	net_set_keep_alive(fd);
 
 	return SLURM_SUCCESS;
 }
@@ -604,14 +626,15 @@ extern int slurm_persist_conn_open(persist_conn_t *persist_conn)
 	 * other side is running yet.
 	 */
 	req_msg.protocol_version = persist_conn->version;
-
-	req_msg.msg_type = tls_enabled() ? REQUEST_PERSIST_INIT_TLS :
-					   REQUEST_PERSIST_INIT;
-
+	req_msg.msg_type = REQUEST_PERSIST_INIT;
 	req_msg.flags |= SLURM_GLOBAL_AUTH_KEY;
 	if (persist_conn->flags & PERSIST_FLAG_DBD)
 		req_msg.flags |= SLURMDBD_CONNECTION;
 	slurm_msg_set_r_uid(&req_msg, persist_conn->r_uid);
+	req_msg.conn = persist_conn->conn;
+
+	if (slurm_conf.debug_flags & DEBUG_FLAG_NET)
+		(void) slurm_msg_t_init_address(&req_msg);
 
 	memset(&req, 0, sizeof(persist_init_req_msg_t));
 	req.cluster_name = persist_conn->cluster_name;
@@ -621,22 +644,16 @@ extern int slurm_persist_conn_open(persist_conn_t *persist_conn)
 
 	req_msg.data = &req;
 
-	if (slurm_send_node_msg(persist_conn->fd, &req_msg) < 0) {
+	if (slurm_send_node_msg(persist_conn->conn, &req_msg) < 0) {
 		error("%s: failed to send persistent connection init message to %s:%d",
 		      __func__, persist_conn->rem_host, persist_conn->rem_port);
-		fd_close(&persist_conn->fd);
+		conn_g_destroy(persist_conn->conn, true);
+		persist_conn->conn = NULL;
 	} else {
 		buf_t *buffer = NULL;
 		persist_msg_t msg;
 		persist_conn_t persist_conn_tmp;
 
-		persist_conn->tls_conn = tls_g_create_conn(persist_conn->fd,
-							   TLS_CONN_CLIENT);
-		if (!persist_conn->tls_conn) {
-			error("Failed to enable tls on persistent connection");
-			goto end_it;
-		}
-
 		buffer = _slurm_persist_recv_msg(persist_conn, false);
 
 		if (!buffer) {
@@ -644,7 +661,10 @@ extern int slurm_persist_conn_open(persist_conn_t *persist_conn)
 				error("%s: No response to persist_init",
 				      __func__);
 			}
-			fd_close(&persist_conn->fd);
+
+			conn_g_destroy(persist_conn->conn, true);
+			persist_conn->conn = NULL;
+
 			if (!errno)
 				errno = SLURM_ERROR;
 			goto end_it;
@@ -677,7 +697,8 @@ extern int slurm_persist_conn_open(persist_conn_t *persist_conn)
 				      persist_conn->rem_host,
 				      persist_conn->rem_port);
 			}
-			fd_close(&persist_conn->fd);
+			conn_g_destroy(persist_conn->conn, true);
+			persist_conn->conn = NULL;
 		} else if (resp) {
 			persist_conn->version = resp->ret_info;
 			persist_conn->flags |= resp->flags;
@@ -696,9 +717,8 @@ extern void slurm_persist_conn_close(persist_conn_t *persist_conn)
 	if (!persist_conn)
 		return;
 
-	tls_g_destroy_conn(persist_conn->tls_conn);
-	persist_conn->tls_conn = NULL;
-	fd_close(&persist_conn->fd);
+	conn_g_destroy(persist_conn->conn, true);
+	persist_conn->conn = NULL;
 }
 
 extern int slurm_persist_conn_reopen(persist_conn_t *persist_conn)
@@ -745,7 +765,6 @@ extern int slurm_persist_conn_process_msg(persist_conn_t *persist_conn,
 	buf_t *recv_buffer = NULL;
 	char *comment = NULL;
 	bool init_msg = false;
-	tls_conn_mode_t tls_mode = TLS_CONN_NULL;
 
 	/* puts msg_char into buffer struct */
 	recv_buffer = create_buf(msg_char, msg_size);
@@ -757,60 +776,40 @@ extern int slurm_persist_conn_process_msg(persist_conn_t *persist_conn,
 				     * (done later in this
 				     * function). */
 
-	if (persist_msg->msg_type == REQUEST_PERSIST_INIT_TLS)
-		tls_mode = TLS_CONN_SERVER;
-
 	if (rc != SLURM_SUCCESS) {
 		comment = xstrdup_printf("Failed to unpack %s message",
 					 slurmdbd_msg_type_2_str(
 						 persist_msg->msg_type, true));
-		error("CONN:%u %s", persist_conn->fd, comment);
+		error("CONN:%u %s",
+		      conn_g_get_fd(persist_conn->conn), comment);
 		*out_buffer = slurm_persist_make_rc_msg(
 			persist_conn, rc, comment, persist_msg->msg_type);
 		xfree(comment);
 
-		/*
-		 * Need the tls_conn to send back the error message.
-		 * There is a chance that the persist_msg->msg_type might not
-		 * have been unpacked, in which case tls_mode could be
-		 * TLS_CONN_NULL when it should be TLS_CONN_SERVER.
-		 */
-		if (!persist_conn->tls_conn) {
-			persist_conn->tls_conn =
-				tls_g_create_conn(persist_conn->fd, tls_mode);
-			if (!persist_conn->tls_conn)
-				error("CONN:%u tls_g_create_conn() failed",
-				      persist_conn->fd);
-		}
-
 		return rc;
 	}
 
-	if ((persist_msg->msg_type == REQUEST_PERSIST_INIT) ||
-	    (persist_msg->msg_type == REQUEST_PERSIST_INIT_TLS))
+	if (persist_msg->msg_type == REQUEST_PERSIST_INIT)
 		init_msg = true;
 
 	if (first && !init_msg) {
 		comment = "Initial RPC not REQUEST_PERSIST_INIT";
 		error("CONN:%u %s type (%d)",
-		      persist_conn->fd, comment, persist_msg->msg_type);
+		      conn_g_get_fd(persist_conn->conn), comment,
+		      persist_msg->msg_type);
 		rc = EINVAL;
 		*out_buffer = slurm_persist_make_rc_msg(
 			persist_conn, rc, comment,
 			REQUEST_PERSIST_INIT);
 	} else if (!first && init_msg) {
 		comment = "REQUEST_PERSIST_INIT sent after connection established";
-		error("CONN:%u %s", persist_conn->fd, comment);
+		error("CONN:%u %s",
+		      conn_g_get_fd(persist_conn->conn),
+		      comment);
 		rc = EINVAL;
-		*out_buffer = slurm_persist_make_rc_msg(
-			persist_conn, rc, comment, REQUEST_PERSIST_INIT);
-	} else if (init_msg) {
-		persist_conn->tls_conn = tls_g_create_conn(persist_conn->fd,
-							   tls_mode);
-		if (!persist_conn->tls_conn) {
-			error("CONN:%u tls_g_create_conn() failed", persist_conn->fd);
-			rc = EINVAL;
-		}
+		*out_buffer =
+			slurm_persist_make_rc_msg(persist_conn, rc, comment,
+						  REQUEST_PERSIST_INIT);
 	}
 
 	return rc;
@@ -828,30 +827,35 @@ extern int slurm_persist_conn_writeable(persist_conn_t *persist_conn)
 	int rc, time_left;
 	struct timeval tstart;
 	char temp[2];
+	int fd;
 
 	if (!persist_conn || !persist_conn->shutdown)
 		fatal("%s: unexpected NULL persist_conn", __func__);
-	else if (*persist_conn->shutdown) {
-		log_flag(NET, "%s: called on shutdown fd:%d to host %s:%hu",
-		         __func__, persist_conn->fd,
-		         (persist_conn->rem_host ? persist_conn->rem_host :
-                                                   "unknown"),
-		         persist_conn->rem_port);
+
+	if (!persist_conn->conn) {
+		log_flag(NET, "%s: called on invalid connection to host %s:%hu",
+		         __func__, (persist_conn->rem_host ?
+				    persist_conn->rem_host :
+				    "unknown"),
+			 persist_conn->rem_port);
 		return -1;
-	} else if (persist_conn->fd < 0) {
-		log_flag(NET, "%s: called on invalid fd:%d to host %s:%hu",
-		         __func__, persist_conn->fd,
-		         (persist_conn->rem_host ? persist_conn->rem_host :
-                                                   "unknown"),
+	}
+	fd = conn_g_get_fd(persist_conn->conn);
+
+	if (*persist_conn->shutdown) {
+		log_flag(NET, "%s: called on shutdown fd:%d to host %s:%hu",
+		         __func__, fd, (persist_conn->rem_host ?
+					persist_conn->rem_host :
+					"unknown"),
 		         persist_conn->rem_port);
 		return -1;
 	}
 
-	ufds.fd     = persist_conn->fd;
+	ufds.fd = fd;
 	ufds.events = POLLOUT;
 	gettimeofday(&tstart, NULL);
 	while (!*persist_conn->shutdown) {
-		time_left = write_timeout - _tot_wait(&tstart);
+		time_left = write_timeout - timeval_tot_wait(&tstart);
 		rc = poll(&ufds, 1, time_left);
 		if (rc == -1) {
 			if ((errno == EINTR) || (errno == EAGAIN))
@@ -869,29 +873,29 @@ extern int slurm_persist_conn_writeable(persist_conn_t *persist_conn)
 		 * nonblocking read means just that.
 		 */
 		if (ufds.revents & POLLHUP ||
-		    (recv(persist_conn->fd, &temp, 1, 0) == 0)) {
+		    (recv(ufds.fd, &temp, 1, 0) == 0)) {
 			log_flag(NET, "%s: persistent connection %d is closed for writes",
-				 __func__, persist_conn->fd);
+				 __func__, ufds.fd);
 			if (persist_conn->trigger_callbacks.dbd_fail)
 				(persist_conn->trigger_callbacks.dbd_fail)();
+			conn_g_set_graceful_shutdown(persist_conn->conn, false);
 			return -1;
 		}
 		if (ufds.revents & POLLNVAL) {
 			error("%s: persistent connection %d is invalid",
-			      __func__, persist_conn->fd);
+			      __func__, ufds.fd);
 			return 0;
 		}
 		if (ufds.revents & POLLERR) {
 			if (_comm_fail_log(persist_conn)) {
 				int rc, err;
-				if ((rc = fd_get_socket_error(persist_conn->fd,
-							      &err)))
+				if ((rc = fd_get_socket_error(ufds.fd, &err)))
 					error("%s: unable to get error for persistent connection %d: %s",
-					      __func__, persist_conn->fd,
+					      __func__, ufds.fd,
 					      strerror(rc));
 				else
 					error("%s: persistent connection %d experienced an error: %s",
-					      __func__, persist_conn->fd,
+					      __func__, ufds.fd,
 					      strerror(err));
 				errno = err;
 			}
@@ -901,7 +905,7 @@ extern int slurm_persist_conn_writeable(persist_conn_t *persist_conn)
 		}
 		if ((ufds.revents & POLLOUT) == 0) {
 			error("%s: persistent connection %d events %d",
-			      __func__, persist_conn->fd, ufds.revents);
+			      __func__, ufds.fd, ufds.revents);
 			return 0;
 		}
 		/* revents == POLLOUT */
@@ -921,7 +925,7 @@ extern int slurm_persist_send_msg(persist_conn_t *persist_conn,
 
 	xassert(persist_conn);
 
-	if (persist_conn->fd < 0)
+	if (!persist_conn->conn)
 		return EAGAIN;
 
 	if (!buffer)
@@ -950,8 +954,7 @@ extern int slurm_persist_send_msg(persist_conn_t *persist_conn,
 	msg_size = get_buf_offset(buffer);
 	nw_size = htonl(msg_size);
 
-	msg_wrote = tls_g_send(persist_conn->tls_conn, &nw_size,
-			       sizeof(nw_size));
+	msg_wrote = conn_g_send(persist_conn->conn, &nw_size, sizeof(nw_size));
 	if (msg_wrote != sizeof(nw_size))
 		return EAGAIN;
 
@@ -962,7 +965,7 @@ extern int slurm_persist_send_msg(persist_conn_t *persist_conn,
 			goto re_open;
 		if (rc < 1)
 			return EAGAIN;
-		msg_wrote = tls_g_send(persist_conn->tls_conn, msg, msg_size);
+		msg_wrote = conn_g_send(persist_conn->conn, msg, msg_size);
 		if (msg_wrote <= 0)
 			return EAGAIN;
 		msg += msg_wrote;
@@ -982,10 +985,10 @@ static buf_t *_slurm_persist_recv_msg(persist_conn_t *persist_conn,
 
 	xassert(persist_conn);
 
-	if (persist_conn->fd < 0) {
+	if (!persist_conn->conn) {
 		if (!persist_conn->shutdown || *persist_conn->shutdown)
-			log_flag(NET, "%s: Invalid file descriptor fd:%d host:%s port:%u",
-				 __func__, persist_conn->fd,
+			log_flag(NET, "%s: Invalid connection to host:%s port:%u",
+				 __func__,
 				 persist_conn->rem_host,
 				 persist_conn->rem_port);
 		return NULL;
@@ -993,12 +996,11 @@ static buf_t *_slurm_persist_recv_msg(persist_conn_t *persist_conn,
 
 	if (!_conn_readable(persist_conn)) {
 		log_flag(NET, "%s: Unable to read from file descriptor (%d)",
-			 __func__, persist_conn->fd);
+			 __func__, conn_g_get_fd(persist_conn->conn));
 		goto endit;
 	}
 
-	msg_read = tls_g_recv(persist_conn->tls_conn, &nw_size,
-			      sizeof(nw_size));
+	msg_read = conn_g_recv(persist_conn->conn, &nw_size, sizeof(nw_size));
 	if (msg_read != sizeof(nw_size)) {
 		log_flag(NET, "%s: Unable to read message size: only read %zd bytes of expected %zu.",
 			 __func__, msg_read, sizeof(nw_size));
@@ -1023,11 +1025,12 @@ static buf_t *_slurm_persist_recv_msg(persist_conn_t *persist_conn,
 	while (msg_size > offset) {
 		if (!_conn_readable(persist_conn))
 			break;		/* problem with this socket */
-		msg_read = tls_g_recv(persist_conn->tls_conn, (msg + offset),
-				      (msg_size - offset));
+		msg_read = conn_g_recv(persist_conn->conn, (msg + offset),
+				       (msg_size - offset));
 		if (msg_read <= 0) {
 			error("%s: read of fd %u failed: %m",
-			      __func__, persist_conn->fd);
+			      __func__,
+			      conn_g_get_fd(persist_conn->conn));
 			break;
 		}
 		offset += msg_read;
@@ -1128,8 +1131,7 @@ extern int slurm_persist_msg_unpack(persist_conn_t *persist_conn,
 	 * future we need to use it in some way to verify things for messages
 	 * that don't have on that will follow on the connection.
 	 */
-	if ((resp_msg->msg_type == REQUEST_PERSIST_INIT) ||
-	    (resp_msg->msg_type == REQUEST_PERSIST_INIT_TLS)) {
+	if (resp_msg->msg_type == REQUEST_PERSIST_INIT) {
 		slurm_msg_t *msg = resp_msg->data;
 		if (persist_conn->auth_cred)
 			auth_g_destroy(persist_conn->auth_cred);
