@@ -90,7 +90,6 @@ static char *_expand_mult(char *list, char *type, int *error_code)
 {
 	char *ast, *end_ptr = NULL, *result = NULL, *save_ptr = NULL;
 	char *sep = "", *tmp, *tok;
-	long int count, i;
 	int (*isfunc) (int) = isdigit;
 	bool mask = false;
 
@@ -108,6 +107,8 @@ static char *_expand_mult(char *list, char *type, int *error_code)
 
 	tok = strtok_r(tmp, ",", &save_ptr);
 	while (tok) {
+		long int i;
+		long int count = 1;
 		if (mask && !xstrncmp(tok, "0x", 2))
 			tok+=2;
 
@@ -132,26 +133,18 @@ static char *_expand_mult(char *list, char *type, int *error_code)
 				break;
 			}
 			ast[0] = '\0';
-			for (int i = 0; tok[i]; i++)
-				if (!isfunc(tok[i])) {
-					error("Failed to validate number: %s, the offending character is %c",
-					      tok, tok[i]);
-					*error_code = SLURM_ERROR;
-					return 0;
-				}
+		}
 
-			for (i = 0; i < count; i++) {
-				xstrfmtcat(result, "%s%s", sep, tok);
-				sep = ",";
+		for (i = 0; tok[i]; i++) {
+			if (!isfunc(tok[i])) {
+				error("Failed to validate number: %s, the offending character is %c",
+				      tok, tok[i]);
+				*error_code = SLURM_ERROR;
+				return 0;
 			}
-		} else {
-			for (int i = 0; tok[i]; i++)
-				if (!isfunc(tok[i])) {
-					error("Failed to validate number: %s, the offending character is %c",
-					      tok, tok[i]);
-					*error_code = SLURM_ERROR;
-					return 0;
-				}
+		}
+
+		for (i = 0; i < count; i++) {
 			xstrfmtcat(result, "%s%s", sep, tok);
 			sep = ",";
 		}
@@ -159,6 +152,11 @@ static char *_expand_mult(char *list, char *type, int *error_code)
 	}
 	xfree(tmp);
 
+	if (!result) {
+		error("Failed to expand list: '%s'", list);
+		*error_code = SLURM_ERROR;
+	}
+
 	return result;
 }
 
@@ -196,8 +194,6 @@ void slurm_sprint_cpu_bind_type(char *str, cpu_bind_type_t cpu_bind_type)
 		strcat(str, "ldoms,");
 	if (cpu_bind_type & CPU_BIND_NONE)
 		strcat(str, "none,");
-	if (cpu_bind_type & CPU_BIND_RANK)
-		strcat(str, "rank,");
 	if (cpu_bind_type & CPU_BIND_MAP)
 		strcat(str, "map_cpu,");
 	if (cpu_bind_type & CPU_BIND_MASK)
@@ -243,8 +239,6 @@ extern char *slurm_xstr_mem_bind_type(mem_bind_type_t mem_bind_type)
 		xstrcat(str, "verbose,");
 	if (mem_bind_type & MEM_BIND_PREFER)
 		xstrcat(str, "prefer,");
-	if (mem_bind_type & MEM_BIND_SORT)
-		xstrcat(str, "sort,");
 	if (mem_bind_type & MEM_BIND_NONE)
 		xstrcat(str, "none,");
 	if (mem_bind_type & MEM_BIND_RANK)
@@ -275,7 +269,6 @@ void slurm_print_cpu_bind_help(void)
 "        q[uiet]         quietly bind before task runs (default)\n"
 "        v[erbose]       verbosely report binding before task runs\n"
 "        no[ne]          don't bind tasks to CPUs (default)\n"
-"        rank            bind by task rank\n"
 "        map_cpu:<list>  specify a CPU ID binding for each task\n"
 "                        where <list> is <cpuid1>,<cpuid2>,...<cpuidN>\n"
 "        mask_cpu:<list> specify a CPU ID binding mask for each task\n"
@@ -320,9 +313,8 @@ extern int slurm_verify_cpu_bind(const char *arg, char **cpu_bind,
 	bool log_binding = true;
 	int rc = SLURM_SUCCESS;
 
-	bind_bits = CPU_BIND_NONE | CPU_BIND_RANK | CPU_BIND_MAP |
-		    CPU_BIND_MASK | CPU_BIND_LDRANK | CPU_BIND_LDMAP |
-		    CPU_BIND_LDMASK;
+	bind_bits = CPU_BIND_NONE | CPU_BIND_MAP | CPU_BIND_MASK |
+		    CPU_BIND_LDRANK | CPU_BIND_LDMAP | CPU_BIND_LDMASK;
 	bind_to_bits = CPU_BIND_TO_SOCKETS | CPU_BIND_TO_CORES |
 		       CPU_BIND_TO_THREADS | CPU_BIND_TO_LDOMS;
 
@@ -360,7 +352,7 @@ extern int slurm_verify_cpu_bind(const char *arg, char **cpu_bind,
 			_clear_then_set((int *)flags, bind_bits, CPU_BIND_NONE);
 			xfree(*cpu_bind);
 		} else if (xstrcasecmp(tok, "rank") == 0) {
-			_clear_then_set((int *)flags, bind_bits, CPU_BIND_RANK);
+			info("Ignoring --cpu-bind=rank. Rank binding is obsolete.");
 			xfree(*cpu_bind);
 		} else if ((xstrncasecmp(tok, "map_cpu", 7) == 0) ||
 		           (xstrncasecmp(tok, "mapcpu", 6) == 0)) {
@@ -598,10 +590,6 @@ int slurm_verify_mem_bind(const char *arg, char **mem_bind,
 		} else if ((xstrcasecmp(tok, "p") == 0) ||
 			   (xstrcasecmp(tok, "prefer") == 0)) {
 		        *flags |= MEM_BIND_PREFER;
-		} else if (!xstrcasecmp(tok, "nosort")) {
-		        *flags &= ~MEM_BIND_SORT;
-		} else if (!xstrcasecmp(tok, "sort")) {
-		        *flags |= MEM_BIND_SORT;
 		} else if ((xstrcasecmp(tok, "q") == 0) ||
 			   (xstrcasecmp(tok, "quiet") == 0)) {
 		        *flags &= ~MEM_BIND_VERBOSE;
