@@ -120,19 +120,15 @@ typedef struct {
 	gid_t gid;
 } auth_credential_t;
 
-/*
- * init() is called when the plugin is loaded, before any other functions
- * are called.  Put global initialization here.
- */
 extern int init(void)
 {
 	debug("%s loaded", plugin_name);
 	return SLURM_SUCCESS;
 }
 
-extern int fini(void)
+extern void fini(void)
 {
-	return SLURM_SUCCESS;
+	return;
 }
 
 /*
@@ -180,32 +176,16 @@ int auth_p_verify(auth_credential_t *cred, char *auth_info)
 	return SLURM_SUCCESS;
 }
 
-/*
- * Obtain the Linux UID from the credential.  The accuracy of this data
- * is not assured until auth_p_verify() has been called for it.
- */
-uid_t auth_p_get_uid(auth_credential_t *cred)
+extern void auth_p_get_ids(auth_credential_t *cred, uid_t *uid, gid_t *gid)
 {
 	if (!cred) {
-		slurm_seterrno(ESLURM_AUTH_BADARG);
-		return SLURM_AUTH_NOBODY;
+		*uid = SLURM_AUTH_NOBODY;
+		*gid = SLURM_AUTH_NOBODY;
+		return;
 	}
 
-	return cred->uid;
-}
-
-/*
- * Obtain the Linux GID from the credential.
- * See auth_p_get_uid() above for details on correct behavior.
- */
-gid_t auth_p_get_gid(auth_credential_t *cred)
-{
-	if (!cred) {
-		slurm_seterrno(ESLURM_AUTH_BADARG);
-		return SLURM_AUTH_NOBODY;
-	}
-
-	return cred->gid;
+	*uid = cred->uid;
+	*gid = cred->gid;
 }
 
 /*
@@ -215,7 +195,7 @@ gid_t auth_p_get_gid(auth_credential_t *cred)
 char *auth_p_get_host(auth_credential_t *cred)
 {
 	if (!cred) {
-		slurm_seterrno(ESLURM_AUTH_BADARG);
+		errno = ESLURM_AUTH_BADARG;
 		return NULL;
 	}
 
@@ -225,7 +205,7 @@ char *auth_p_get_host(auth_credential_t *cred)
 extern int auth_p_get_data(auth_credential_t *cred, char **data, uint32_t *len)
 {
 	if (!cred) {
-		slurm_seterrno(ESLURM_AUTH_BADARG);
+		errno = ESLURM_AUTH_BADARG;
 		return SLURM_ERROR;
 	}
 
@@ -235,6 +215,16 @@ extern int auth_p_get_data(auth_credential_t *cred, char **data, uint32_t *len)
 	return SLURM_SUCCESS;
 }
 
+extern void *auth_p_get_identity(auth_credential_t *cred)
+{
+	if (!cred) {
+		errno = ESLURM_AUTH_BADARG;
+		return NULL;
+	}
+
+	return NULL;
+}
+
 /*
  * Marshall a credential for transmission over the network, according to
  * Slurm's marshalling protocol.
@@ -242,13 +232,13 @@ extern int auth_p_get_data(auth_credential_t *cred, char **data, uint32_t *len)
 int auth_p_pack(auth_credential_t *cred, buf_t *buf, uint16_t protocol_version)
 {
 	if (!cred || !buf) {
-		slurm_seterrno(ESLURM_AUTH_BADARG);
+		errno = ESLURM_AUTH_BADARG;
 		return SLURM_ERROR;
 	}
 
 	if (protocol_version >= SLURM_MIN_PROTOCOL_VERSION) {
-		pack32((uint32_t) cred->uid, buf);
-		pack32((uint32_t) cred->gid, buf);
+		pack32(cred->uid, buf);
+		pack32(cred->gid, buf);
 		packstr(cred->hostname, buf);
 	} else {
 		error("%s: Unknown protocol version %d",
@@ -266,11 +256,9 @@ int auth_p_pack(auth_credential_t *cred, buf_t *buf, uint16_t protocol_version)
 auth_credential_t *auth_p_unpack(buf_t *buf, uint16_t protocol_version)
 {
 	auth_credential_t *cred = NULL;
-	uint32_t tmpint;
-	uint32_t uint32_tmp = 0;
 
 	if (!buf) {
-		slurm_seterrno(ESLURM_AUTH_BADARG);
+		errno = ESLURM_AUTH_BADARG;
 		return NULL;
 	}
 
@@ -278,20 +266,9 @@ auth_credential_t *auth_p_unpack(buf_t *buf, uint16_t protocol_version)
 	cred = xmalloc(sizeof(*cred));
 
 	if (protocol_version >= SLURM_MIN_PROTOCOL_VERSION) {
-		/*
-		 * We do it the hard way because we don't know anything about
-		 * the size of uid_t or gid_t, only that they are integer
-		 * values.  We pack them as 32-bit integers, but we can't pass
-		 * addresses to them directly to unpack as 32-bit integers
-		 * because there will be bad clobbering if they really aren't.
-		 * This technique ensures a warning at compile time if the sizes
-		 * are incompatible.
-		 */
-		safe_unpack32(&tmpint, buf);
-		cred->uid = tmpint;
-		safe_unpack32(&tmpint, buf);
-		cred->gid = tmpint;
-		safe_unpackstr_xmalloc(&cred->hostname, &uint32_tmp, buf);
+		safe_unpack32(&cred->uid, buf);
+		safe_unpack32(&cred->gid, buf);
+		safe_unpackstr(&cred->hostname, buf);
 	} else {
 		error("%s: unknown protocol version %u",
 		      __func__, protocol_version);
@@ -302,7 +279,7 @@ auth_credential_t *auth_p_unpack(buf_t *buf, uint16_t protocol_version)
 
 unpack_error:
 	auth_p_destroy(cred);
-	slurm_seterrno(ESLURM_AUTH_UNPACK);
+	errno = ESLURM_AUTH_UNPACK;
 	return NULL;
 }
 
@@ -321,3 +298,8 @@ char *auth_p_token_generate(const char *username, int lifespan)
 {
 	return NULL;
 }
+
+extern int auth_p_get_reconfig_fd(void)
+{
+	return -1;
+}
