@@ -1489,6 +1489,8 @@ static list_t *_get_system_gpu_list_nvml(node_config_load_t *node_config)
 			if (mig_count == 0)
 				error("MIG mode is enabled, but no MIG devices were found. Please either create MIG instances, disable MIG mode, remove AutoDetect=nvml, or remove GPUs from the configuration completely.");
 
+			gres_slurmd_conf.config_flags |= GRES_CONF_MIG;
+
 			for (unsigned int j = 0; j < mig_count; j++) {
 				nvml_mig_t nvml_mig = { 0 };
 				nvml_mig.files = xstrdup(device_file);
@@ -1528,6 +1530,7 @@ static list_t *_get_system_gpu_list_nvml(node_config_load_t *node_config)
 			gres_slurmd_conf.file = device_file;
 			gres_slurmd_conf.links = nvlinks;
 			gres_slurmd_conf.type_name = device_name;
+			gres_slurmd_conf.unique_id = uuid;
 
 			add_gres_to_list(gres_list_system, &gres_slurmd_conf);
 		}
@@ -1701,13 +1704,11 @@ extern int init(void)
 	return SLURM_SUCCESS;
 }
 
-extern int fini(void)
+extern void fini(void)
 {
 	_nvml_shutdown();
 
 	debug("%s: unloading %s", __func__, plugin_name);
-
-	return SLURM_SUCCESS;
 }
 
 extern list_t *gpu_p_get_system_gpu_list(node_config_load_t *node_config)
