@@ -124,7 +124,7 @@ static void  _handle_sigwinch(int sig)
 	xsignal(SIGWINCH, _handle_sigwinch);
 }
 
-static void _notify_winsize_change(void *tls_conn, srun_job_t *job)
+static void _notify_winsize_change(void *conn, srun_job_t *job)
 {
 	pty_winsz_t winsz;
 	int len;
@@ -134,14 +134,14 @@ static void _notify_winsize_change(void *tls_conn, srun_job_t *job)
 	winsz.rows = htons(job->ws_row);
 	memcpy(buf, &winsz.cols, 2);
 	memcpy(buf+2, &winsz.rows, 2);
-	len = slurm_write_stream(tls_conn, buf, 4);
+	len = slurm_write_stream(conn, buf, 4);
 	if (len < sizeof(winsz))
 		error("pty: window size change notification error: %m");
 }
 
 static void *_pty_thread(void *arg)
 {
-	void *tls_conn = NULL;
+	void *conn = NULL;
 	int fd = -1;
 	srun_job_t *job = (srun_job_t *) arg;
 	slurm_addr_t client_addr;
@@ -150,7 +150,7 @@ static void *_pty_thread(void *arg)
 	xsignal_unblock(pty_sigarray);
 	xsignal(SIGWINCH, _handle_sigwinch);
 
-	if (!(tls_conn = slurm_accept_msg_conn(job->pty_fd, &client_addr))) {
+	if (!(conn = slurm_accept_msg_conn(job->pty_fd, &client_addr))) {
 		error("pty: accept failure: %m");
 		return NULL;
 	}
@@ -160,7 +160,7 @@ static void *_pty_thread(void *arg)
 	cfmakeraw(&term);
 	tcsetattr(job->input_fd, TCSANOW, &term);
 
-	fd = conn_g_get_fd(tls_conn);
+	fd = conn_g_get_fd(conn);
 
 	net_set_keep_alive(fd);
 	while (job->state <= SRUN_JOB_RUNNING) {
@@ -171,11 +171,11 @@ static void *_pty_thread(void *arg)
 		}
 		if (winch) {
 			set_winsize(STDOUT_FILENO, job);
-			_notify_winsize_change(tls_conn, job);
+			_notify_winsize_change(conn, job);
 		}
 		winch = 0;
 	}
 
-	conn_g_destroy(tls_conn, true);
+	conn_g_destroy(conn, true);
 	return NULL;
 }
